# -*- coding: utf-8 -*-

# from qtpy import QtMultimedia
# from prettyqt import multimedia

# QtMultimedia.QAbstractPlanarVideoBuffer.__bases__ = (multimedia.AbstractVideoBuffer,)


# class AbstractPlanarVideoBuffer(QtMultimedia.QAbstractPlanarVideoBuffer):
#     pass
