# -*- coding: utf-8 -*-

"""quick module.

contains QtQuick-based classes
"""

from .quickview import QuickView
from .quickitem import QuickItem
from .quickpainteditem import QuickPaintedItem
from .quickwindow import QuickWindow


__all__ = ["QuickView", "QuickItem", "QuickWindow", "QuickPaintedItem"]
