# # -*- coding: utf-8 -*-

# from qtpy import QtCore

# from prettyqt import core


# QtCore.QRandomGenerator64.__bases__ = (core.RandomGenerator,)


# class RandomGenerator64(QtCore.QRandomGenerator64):
#     pass
